
#include "P2_1_aliases.h"
#include "P2_1.h"
#include "CAN.h"

/*******************************************************************************
* User-defined parameters
********************************************************************************/
/* CAN comm. low-level parameters */
#define CAN_TX_DELAY 				(1u)   	/* This is a delay between CAN Tx retries */
#define MAX_CAN_TX_RETRIES   		(0xFF)   /* This parameter determines number of CAN Tx retries */
#define WAIT_FOR_EOP					(100)		/* /us/ Delay to wait for to ensure a complete Bootload Command is received */

/* Bootloader comm. timeout increment */
#define TIMEOUT_DELAY_INCREMENT	(100)		/* /us/ CommRead function has a timeout. This parameter defines duration of delay between checking the comm channel */

#define DEVICE_ID 					(0x02)	/* Arbitrary definition of device ID - not used in the bootloader */

#define CAN_RX_BUFFER_SIZE    	(8)		/* This size is determined by the largest command sent from the host */

/*******************************************************************************
* Constants
********************************************************************************/
#define MAILBOX0						(0)
#define BTLDR_EOP 					(0x17)	/* Bootloader end-of-packet is 0x17 (see Fig. 20, AN68272) */

#define DATA_PACKET_NO_OF_BYTES (0x118) //280

#define PACKET_BYTES_WITHOUT_DATA (0x7)
#define PACKET_SEND_DATA_BYTES_NO (0x11F) //287



/*******************************************************************************
* CAN Bootloader internal variables and buffers
********************************************************************************/
uint8 XDATA CAN_RxBuffer[CAN_RX_BUFFER_SIZE];	/* CAN peripheral read buffer - i.e. Host writes commands to this buffer */
volatile uint8 RxBufIndex;
volatile uint8 CAN_StartBootloadMsg;
uint8 msg_count;											/* this variable will hold the number of complete 8-byte segments in a transaction */



uint8 completePacketCodeReceived = 0; //it becomes 1 when a 0x17 is received within at the end of specific message (package end)
uint16 dataIndexCntr = 0;   //hold the index for storing data into memory
uint16 newCountForDataPacket = 0; // countHowManyBytes were received

/*******************************************************************************
* Function Name: CyBtldrCommStart
********************************************************************************
*
* Summary:
*  Starts the CAN communication component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Side Effects:
*  This component automatically enables global interrupt.
*
*******************************************************************************/
void CyBtldrCommStart(void)  
{
   /* **** CAN component already started in main **** */
   
   RxBufIndex = 0;
	
   /* clear CAN receive mailbox registers - arbitrary choice to use mailbox 2 */
	/* note - CY_SET_REG32 is a macro defined in cytypes.h and CYDEV_CAN0 registers are defined in cydevice.h */
	CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DH, 0u);
    CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DL, 0u);
	
   /* clear CAN transmit mailbox registers - arbitrary choice to use mailbox 2 */
	CY_SET_REG32((reg32 *)CYDEV_CAN0_TX0_DH, 0u);
    CY_SET_REG32((reg32 *)CYDEV_CAN0_TX0_DL, 0u);
}

/*******************************************************************************
* Function Name: CyBtldrCommStop
********************************************************************************
*
* Summary:
*  Disables the communication component and disables the interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void CyBtldrCommStop(void)  
{
   /* Stop CAN component */
   CAN_Stop();
}

/*******************************************************************************
* Function Name: CyBtldrCommReset
********************************************************************************
*
* Summary:
*  Resets the receive and transmit communication Buffers.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void CyBtldrCommReset(void)  
{
   /* Clear RX and TX buffers */
	RxBufIndex = 0;
	
   /* clear CAN receive mailbox registers */
	CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DH, 0u);
    CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DL, 0u);
	
   /* clear CAN transmit mailbox registers */
	CY_SET_REG32((reg32 *)CYDEV_CAN0_TX0_DH, 0u);
    CY_SET_REG32((reg32 *)CYDEV_CAN0_TX0_DL, 0u);
}

/*******************************************************************************
* Function Name: CyBtldrCommWrite
********************************************************************************
*
* Summary:
*  Allows the caller to write data to the bootloader host. This function uses
*  a blocking write function for writing data using CAN. This function should be 
*  called after a bootloader command was received - i.e. to send the reponse
*  packet.
*
* Parameters:
*  pData:    A pointer to the block of data to send to the device
*  size:     The number of bytes to write.
*  count:    Pointer to an uint16 variable to write the number of
*            bytes actually written.
*  timeOut:  Number of units to wait before returning because of a timeout (not used).
*
* Return:
*   cystatus: This function will return CYRET_SUCCESS if data is sent
*             succesfully.
*
* Side Effects:
*  None
*
*******************************************************************************/
cystatus CyBtldrCommWrite(const uint8 pData[], uint16 size, uint16 * count, uint8 timeOut) 
{
    uint16 buf_index = 0u;
	uint8 i, timeoutCAN_Tx = 0;

	/* The timeout is not used in this function, defined to avoid compiler 'unused variable' warning */
    if(timeOut != 0u) { }

   /* Clear receive buffer - this is the time to do it to make sure it's clear before the next command_read from Host */
    RxBufIndex = 0;
	/* clear CAN receive mailbox registers */
	CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DH, 0u);
    CY_SET_REG32((reg32 *)CYDEV_CAN0_RX0_DL, 0u);	

   /* Write TX data using blocking function */
	msg_count = size >> 3;	/* number of full 8-byte messages will be the "size" argument divided by eight ... remainder byte(s) handled below */
    
    while(0u != msg_count)
    {
        for ( i = 0; i < 8; i++ )
		{
			CAN_TX_DATA_BYTE(MAILBOX0, i) = pData[buf_index];	/* write response_buffer contents to CAN Tx mailbox - full 8-byte messages here */
			buf_index++;
		}
      
      /* this is the CAN API to initiate a TX (CAN_TX_RX_func.c) */
        while((CAN_SendMsgBootResponse() != CYRET_SUCCESS) && (timeoutCAN_Tx < MAX_CAN_TX_RETRIES))	
        {
            timeoutCAN_Tx++;
            CyDelayUs(CAN_TX_DELAY); /* delay added to ensure that the previous message is succesfully transmitted */
		}
        timeoutCAN_Tx = 0;

        msg_count--;
    }  /* end of handler for full 8-byte messages */
	
    CyDelayUs(250);
	if(buf_index < size) /* take care of remainder bytes here and add padding bytes if necessary */
	{
        for(i = 0; i < 8; i++)
		{
			if(buf_index < size)
			{
				CAN_TX_DATA_BYTE(MAILBOX0, i) = pData[buf_index];	/* write response_buffer contents to CAN Tx mailbox - leftover byte(s) here */
				buf_index++;
			}
			else
			{
                CAN_TX_DATA_BYTE(MAILBOX0, i) = 0u;	/* write padding bytes (0x00) here to fill out 8-byte message */
		    }
        }
      
      /* this is the CAN API to initiate a TX (CAN_TX_RX_func.c) */
        while((CAN_SendMsgBootResponse() != CYRET_SUCCESS) && (timeoutCAN_Tx < MAX_CAN_TX_RETRIES))	
        {
            timeoutCAN_Tx++;
            CyDelayUs(CAN_TX_DELAY); /* delay added to ensure that the previous message is succesfully transmitted */
		}
        timeoutCAN_Tx = 0;
	   
    } /* end of handler for remainder and padding bytes ... last 8-byte message to send */
	
   /* Return success code */
    *count = size;
	
    return (CYRET_SUCCESS);
}



/*******************************************************************************
* Function Name: CyBtldrCommRead
********************************************************************************
*
* Summary:
*  Receives a command from the host.
*
* Parameters:
*  pData:    A pointer to the area to store the block of data received
*             from the device.
*  size:     Maximum size of the bootloader read buffer
*  count:    Pointer to an unsigned int16 to write the number of bytes 
*            read by the bootloader before writing flash.
*  timeOut:  Number of units to wait before returning because of a timeOut.
*            Timeout is measured in 10s of ms.
*
* Return:
*  cystatus: This function will return CYRET_SUCCESS if atleast one byte is
*            received succesfully within the timeout interval. If no data is
*            received this function will return CYRET_EMPTY.
*
*******************************************************************************/




void writeValueToCAN(uint16 value)
{
    CAN_TX_DATA_BYTE1(1) = value & 0xff;
    CAN_TX_DATA_BYTE2(1) = value >> 8;
    CAN_SendMsgErr();
    CyDelay(1);
}

void writeArrayToCAN(uint8 arrVal[], uint8 startIndex, uint8 endIndex)
{   
    uint8 cn = 0;
    for(uint8 i = startIndex; i< endIndex; i++)
    {
        CAN_TX_DATA_BYTE(1,cn) = arrVal[i];
        if(cn == 7)
        {
            cn = 0;
            CAN_SendMsgErr();
            CyDelay(1);
        }
        else
        {
            cn++;
        }
    }
}




uint8 cnt = 0;
cystatus CyBtldrCommRead(uint8 pData[], uint16 size, uint16* count, uint8 timeOut) 
{
    uint16 iCntr;
   
    uint16 tempCount;
    uint16 oldDataCount;
	uint8 i;
	
    cystatus status = CYRET_EMPTY;
	
   /* Check whether data is received within the timeout period.
    *  Timeout period is in units of 10ms.
    *  If atleast one byte is received within the timeout interval, wait for more data */
    for (iCntr = 0u; iCntr < ((uint16)10u * timeOut); iCntr++)
    {
      /* If atleast one byte is received within the timeout interval
       *  enter the next loop waiting for more data reception
       */
        if(RxBufIndex > 0u)
        {         
             /* 
              * Delay should allow all bytes (till EOP) to get in
              */
            do
            {
                oldDataCount = (RxBufIndex);
               
                CyDelayUs(WAIT_FOR_EOP);   /* Value empirically deduced - depends on delay between successive 8-byte bootload command packets */
            }while(RxBufIndex > oldDataCount); /* if the expression evaluates to TRUE, this indicates more data has arrived ... so stay in the loop a while longer */
             
            status = CYRET_SUCCESS;

            break;
        }
      /* If no data is received, delay for 100us and check again until the timeout specified in Component */
        else
        {
            CyDelayUs(TIMEOUT_DELAY_INCREMENT);   /* Delay upto timeout in 100us increments */
        }
    }

   /* Initialize the data read indexes and Count value */
    *count = 0u;


   /* If RxBufIndex > 0 , move the received data to the pData buffer - remember, CAN_ReceiveMsgBoot_cmd() in CAN_TX_RX_func.c wrote padding bytes to RxBuffer */
    while(RxBufIndex > 0u)
    {
		for(i = 0; i < 8; i++) /* look for real EOP byte (0x17) and discard pad bytes by decrementing/adjusting pointer to EOP position */
	    {
		    if(CAN_RxBuffer[(RxBufIndex - 1)] != BTLDR_EOP) /* start from end of buffer and look backwards for EOP marker */
			{
                RxBufIndex--;	/* adjust pointer and continue looking */
            }
            else
    		{
              
                /* Indicate bootload progress - a complete BTLDR packet has been received */ //16 reprs 1 packet on two bytes
                if(dataIndexCntr < 19 || ((dataIndexCntr > DATA_PACKET_NO_OF_BYTES - RxBufIndex)   && dataIndexCntr < PACKET_SEND_DATA_BYTES_NO ) )
                {
                    completePacketCodeReceived = 1;
                    break;
                }            
    		}		
    	}
	
        tempCount = RxBufIndex;
        *count  =(*count) + tempCount;
        
        
          /* Check if buffer overflow will occur before moving the data */
        if(*count < size)
        {   

        
            if(completePacketCodeReceived)
            {
                for(iCntr = 0u; iCntr < tempCount; iCntr++)
                {
                    /* Read the data and move it to the pData buffer */
                    pData[dataIndexCntr] = CAN_RxBuffer[iCntr];
                    RxBufIndex--;
                    dataIndexCntr++;
                }
                cnt++;
                
                if(newCountForDataPacket >= DATA_PACKET_NO_OF_BYTES || newCountForDataPacket >= 8 )
                {
                
                    *count  = newCountForDataPacket + tempCount;
                    newCountForDataPacket = 0;
                }
                
              
                dataIndexCntr = 0u;
            }

            else
            {   
                newCountForDataPacket += 8;
                *count  = newCountForDataPacket;
                
                for(iCntr = 0u; iCntr < 8; iCntr++)
                {
                    /* Read the data and move it to the pData buffer */
                    pData[dataIndexCntr] = CAN_RxBuffer[iCntr];
                    
                    RxBufIndex--;
                    dataIndexCntr++;
                }
            }
        }
        
       
        
        /* If there is no space to move data, break from the loop */
        else
        {
          
            *count = (*count) - tempCount;
            break;
        }
        
        RxBufIndex = 0;
    }
   
    return (status);
}

/* [] END OF FILE */
