/*******************************************************************************
* File Name: CANx.c
* Version 2.30
*
* Description:
*  The CAN Component provide functionality of Control Area Network.
*  Two types of mailbox configurations available "Full" and "Basic".
*
* Note:
*  CAN configuration puts as constant to ROM and could change only directly
*  registers write by user.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CANx.h"

uint8 CANx_initVar = 0u;


/*******************************************************************************
* FUNCTION NAME:   CANx_Init
********************************************************************************
*
* Summary:
*  Inits/Restores default CAN configuration provided with customizer.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the configuration has been accepted or rejected.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
* Side Effects:
*  All registers will be reset to their initial values. This will re-initialize 
*  the component with the following exceptions - it will not clear data from 
*  the mailboxes.
*  Enable power to the CAN Core.
*
*******************************************************************************/
uint8 CANx_Init(void) 
{
    /* Iniital values of CAN RX and TX registers */
    static const CANx_RX_CFG CYCODE CANx_RXConfigStruct[] =
    {
        { 0u, 0x28u, 0x1FFFF9u, 0x200000u },
        { 1u, 0x28u, 0x1FFFF9u, 0x400000u },
        { 2u, 0x28u, 0x1FFFF9u, 0x600000u },
        { 3u, 0x800088u, 0xFFFFFFFFu, 0x0u },
        { 4u, 0x0u, 0x0u, 0x0u },
        { 5u, 0x0u, 0x0u, 0x0u },
        { 6u, 0x0u, 0x0u, 0x0u },
        { 7u, 0x0u, 0x0u, 0x0u },
        { 8u, 0x0u, 0x0u, 0x0u },
        { 9u, 0x0u, 0x0u, 0x0u },
        { 10u, 0x0u, 0x0u, 0x0u },
        { 11u, 0x0u, 0x0u, 0x0u },
        { 12u, 0x0u, 0x0u, 0x0u },
        { 13u, 0x0u, 0x0u, 0x0u },
        { 14u, 0x0u, 0x0u, 0x0u },
        { 15u, 0x0u, 0x0u, 0x0u }
    };

    static const CANx_TX_CFG CYCODE CANx_TXConfigStruct[] =
    {
        { 0u, 0x80004u, 0x200000u },
        { 1u, 0x80004u, 0x400000u },
        { 2u, 0x10004u, 0x600000u },
        { 3u, 0x880008u, 0x0u },
        { 4u, 0x880008u, 0x0u },
        { 5u, 0x880008u, 0x0u },
        { 6u, 0x880008u, 0x0u },
        { 7u, 0x880008u, 0x0u }
    };
    
    uint8 i;
    uint8 result = CANx_FAIL;
    uint8 localResult = CANx_FAIL;
    uint8 enableInterrupts;
    
    enableInterrupts = CyEnterCriticalSection();
    
    /* Enable CAN block in Active mode */
    CANx_PM_ACT_CFG_REG |= CANx_ACT_PWR_EN;
        
    /* Enable CAN block in Alternate Active (Standby) mode */
    CANx_PM_STBY_CFG_REG |= CANx_STBY_PWR_EN; 
    
    CyExitCriticalSection(enableInterrupts);
    
    /* Sets the CAN controller to stop mode */
    CANx_CMD_REG.byte[0u] &= (uint8) (~CANx_MODE_MASK);

    /* Verify that bit is cleared */
    if ((CANx_CMD_REG.byte[0u] & CANx_MODE_MASK) == 0u)
    {                
        /* Disable Interrupt. */
        CyIntDisable(CANx_ISR_NUMBER);
        
        /* Set the ISR to point to the CANx_ISR Interrupt. */
        (void) CyIntSetVector(CANx_ISR_NUMBER, & CANx_ISR);
        
        /* Set the priority. */
        CyIntSetPriority(CANx_ISR_NUMBER, CANx_ISR_PRIORITY);

        if (CANx_SetPreScaler(CANx_BITRATE) == CYRET_SUCCESS)
        {
            if (CANx_SetArbiter(CANx_ARBITER) == CYRET_SUCCESS)
            {
                if (CANx_SetTsegSample(CANx_CFG_REG_TSEG1, CANx_CFG_REG_TSEG2,
                    CANx_CFG_REG_SJW, CANx_SAMPLING_MODE) == CYRET_SUCCESS)
                {
                    if (CANx_SetRestartType(CANx_RESET_TYPE) == CYRET_SUCCESS)
                    {
                        if (CANx_SetEdgeMode(CANx_SYNC_EDGE) == CYRET_SUCCESS)
                        {
                            localResult = CYRET_SUCCESS;
                        }
                    }
                }
            }
        }
        
        if (localResult == CYRET_SUCCESS)
        {
            /* Initialize TX maiboxes */
            for (i = 0u; i < CANx_NUMBER_OF_TX_MAILBOXES; i++)
            {
                if (CANx_TxBufConfig((const CANx_TX_CFG *)
                    (&CANx_TXConfigStruct[i])) != CYRET_SUCCESS)
                {
                    localResult = CANx_FAIL;
                    break;
                }
            }

            if (localResult == CYRET_SUCCESS)
            {
                /* Initialize RX mailboxes */
                for (i = 0u; i < CANx_NUMBER_OF_RX_MAILBOXES; i++)
                {
                    if (CANx_RxBufConfig((const CANx_RX_CFG *)
                        (&CANx_RXConfigStruct[i])) != CYRET_SUCCESS)
                    {
                        localResult = CANx_FAIL;
                        break;
                    }
                }

                if (localResult == CYRET_SUCCESS)
                {
                    /* Write IRQ Mask */
                    if (CANx_SetIrqMask(CANx_INIT_INTERRUPT_MASK) ==
                        CYRET_SUCCESS)
                    {
                        /* Active mode always */
                        if (CANx_SetOpMode(CANx_ACTIVE_MODE) == CYRET_SUCCESS)
                        {
                            result = CYRET_SUCCESS;
                        }
                    }
                }
            }
        }
                            
    }

    return (result);
}


/*******************************************************************************
* Function Name: CANx_Enable
********************************************************************************
*
* Summary:   
*  This function enable CAN Component and ISR of CAN Component. 
*
* Parameters:  
*  None.
*
* Return: 
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_Enable(void) 
{
    uint8 result = CANx_FAIL;
    uint8 enableInterrupts;
    
    enableInterrupts = CyEnterCriticalSection();
    
    /* Enable CAN block in Active mode */
    CANx_PM_ACT_CFG_REG |= CANx_ACT_PWR_EN;

    /* Enable CAN block in Alternate Active (Standby) mode */
    CANx_PM_STBY_CFG_REG |= CANx_STBY_PWR_EN;
    
    CyExitCriticalSection(enableInterrupts);
    
    /* Enable isr */
    CyIntEnable(CANx_ISR_NUMBER);


    /* Sets the CAN controller to run mode */
    CY_SET_REG32(CANx_CMD_PTR, (CY_GET_REG32(CANx_CMD_PTR) | CANx_MODE_MASK));
    
    /* Verify that bit is set */
    if (0u != (CY_GET_REG32(CANx_CMD_PTR) & CANx_MODE_MASK))
    {
        result = CYRET_SUCCESS;
    }
    
    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_Start
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Run mode. Starts the Rate Counter if
*  polling mailboxes available.
*
* Parameters:
*  None. 
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
* Global variables:
*  CANx_initVar - used to check initial configuration, modified on
*  first function call.
*
* Reentrant:
*  No.
*
*******************************************************************************/
uint8 CANx_Start(void) 
{
    uint8 result = CYRET_SUCCESS;
    
    if (CANx_initVar == 0u)
    {
        result = CANx_Init();
        CANx_initVar = 1u;        
    }
    
    if (result == CYRET_SUCCESS)
    {
        result = CANx_Enable();
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_Stop
********************************************************************************
*
* Summary:
*  This function sets CAN Component into Stop mode.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
* Side Effects:
*  Disable power to the CAN Core.
*
*******************************************************************************/
uint8 CANx_Stop(void) 
{
    uint8 result = CANx_FAIL;
    uint8 enableInterrupts;        
    
    /* Sets the CAN controller to stop mode */
    CY_SET_REG32(CANx_CMD_PTR, (CY_GET_REG32(CANx_CMD_PTR) & 
                                            ((uint8) (~CANx_MODE_MASK))));

    /* Verify that bit is cleared */
    if (0u == (CY_GET_REG32(CANx_CMD_PTR) & CANx_MODE_MASK))
    {
        result = CYRET_SUCCESS;
    }
    
    /* Disable isr */
    CyIntDisable(CANx_ISR_NUMBER);

    enableInterrupts = CyEnterCriticalSection(); 

    /* Disable CAN block in Active mode */
    CANx_PM_ACT_CFG_REG &= (uint8) (~CANx_ACT_PWR_EN);

    /* Disable CAN block in Alternate Active (Standby) mode template */
    CANx_PM_STBY_CFG_REG &= (uint8) (~CANx_STBY_PWR_EN);
    
    CyExitCriticalSection(enableInterrupts);

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_GlobalIntEnable
********************************************************************************
*
* Summary:
*  This function enables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_GlobalIntEnable(void) 
{
    uint8 result = CANx_FAIL;

    CANx_INT_EN_REG.byte[0u] |= CANx_GLOBAL_INT_MASK;

    /* Verify that bit is set */
    if ((CANx_INT_EN_REG.byte[0u] & CANx_GLOBAL_INT_MASK) == CANx_GLOBAL_INT_MASK)
    {
        result = CYRET_SUCCESS;
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_GlobalIntDisable
********************************************************************************
*
* Summary:
*  This function disables Global Interrupts from CAN Core.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_GlobalIntDisable(void) 
{
    uint8 result = CANx_FAIL;

    CANx_INT_EN_REG.byte[0u] &= (uint8) (~CANx_GLOBAL_INT_MASK);

    /* Verify that bit is cleared */
    if ((CANx_INT_EN_REG.byte[0u] & CANx_GLOBAL_INT_MASK) == 0u)
    {
        result = CYRET_SUCCESS;
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetPreScaler
********************************************************************************
*
* Summary:
*  This function sets PreScaler for generation the time quantum which defines
*  the time quanta. Value between 0x0 and 0x7FFF are valid.
*
* Parameters:
*  bitrate: PreScaler value.
*   Value           Description
*    0               One time quantum equals 1 clock cycle
*    1               One time quantum equals 2 clock cycles
*    ...             ...
*    32767           One time quantum equals 32768 clock cycles
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*    CANx_OUT_OF_RANGE      Function parameter is out of range
*
*******************************************************************************/
uint8 CANx_SetPreScaler(uint16 bitrate) 
{
    uint8 result = CANx_OUT_OF_RANGE;

    if (bitrate <= CANx_BITRATE_MASK)
    {
        result = CANx_FAIL;

        /* Set prescaler */
        CY_SET_REG16((reg16 *) &CANx_CFG_REG.byte[2u], bitrate);

        /* Verify that prescaler is set */
        if (CY_GET_REG16((reg16 *) &CANx_CFG_REG.byte[2u]) == bitrate)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetArbiter
********************************************************************************
*
* Summary:
*  This function sets arbitration type for transmit mailboxes. Types of
*  arbiters are Round Robin and Fixed priority. Value 0 and 1 are valid.
*
* Parameters:
*  arbiter: Arbitaration type for transmit mailboxes.
*   Value                              Description
*    CANx_ROUND_ROBIN       Round robin arbitration
*    CANx_FIXED_PRIORITY    Fixed priority arbitration.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_SetArbiter(uint8 arbiter) 
{
    uint8 result = CANx_FAIL;

    if (arbiter == CANx_ROUND_ROBIN)
    {
        CANx_CFG_REG.byte[1u] &= (uint8) (~CANx_ARBITRATION_MASK);    /* Round Robin */

        /* Verify that bit is cleared */
        if ((CANx_CFG_REG.byte[1u] & CANx_ARBITRATION_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }        
    }
    else
    {
        CANx_CFG_REG.byte[1u] |= CANx_ARBITRATION_MASK;    /* Fixed priority */

        /* Verify that bit is set */
        if ((CANx_CFG_REG.byte[1u] & CANx_ARBITRATION_MASK) ==
            CANx_ARBITRATION_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME: CANx_SetTsegSample
********************************************************************************
*
* Summary:
*  This function configures: Time segment 1, Time segment 2, Sampling Mode
*  and Synchronization Jump Width.
*
* Parameters:
*  cfgTseg1: Length of time segment 1, value between 0x2 and 0xF are valid;
*  cfgTseg2: Length of time segment 2, value between 0x1 and 0x7 are valid;
*  sjw: Synchronization Jump Width, value between 0x0 and 0x3 are valid;
*  sm: Sampling Mode.
*   Define                               Description
*   CANx_ONE_SAMPLE_POINT     one sampling points is used
*   CANx_THREE_SAMPLE_POINTS  three sampling points are used
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*    CANx_OUT_OF_RANGE      Function parameter is out of range
*
*******************************************************************************/
uint8 CANx_SetTsegSample(uint8 cfgTseg1, uint8 cfgTseg2, uint8 sjw, uint8 sm)
                                     
{
    uint8 result = CANx_OUT_OF_RANGE;
    uint8 cfgTemp;

    if ((cfgTseg1 >= CANx_CFG_REG_TSEG1_LOWER_LIMIT) && (cfgTseg1 <=
         CANx_CFG_REG_TSEG1_UPPER_LIMIT))
    {
        if (((cfgTseg2 >= CANx_CFG_REG_TSEG2_LOWER_LIMIT) &&
             (cfgTseg2 <= CANx_CFG_REG_TSEG2_UPPER_LIMIT)) || ((sm == CANx_ONE_SAMPLE_POINT) &&
             (cfgTseg2 == CANx_CFG_REG_TSEG2_EXCEPTION)))
        {
            if ((sjw <= CANx_CFG_REG_SJW_LOWER_LIMIT) && (sjw <= cfgTseg1) && (sjw <= cfgTseg2))
            {
                result = CANx_FAIL;

                cfgTemp = CANx_CFG_REG.byte[1];
                cfgTemp &= (uint8) (~CANx_CFG_REG_TSEG1_MASK);
                cfgTemp |= cfgTseg1;

                /* write register byte 1 */
                CANx_CFG_REG.byte[1u] = cfgTemp;

                /* Verify 2nd byte of CANx_CFG_REG register */
                if (CANx_CFG_REG.byte[1u] == cfgTemp)
                {
                    cfgTemp = 0u;
                    /* Set appropriate bits */
                    if (sm != CANx_ONE_SAMPLE_POINT)
                    {
                        cfgTemp = CANx_SAMPLE_MODE_MASK;
                    }
                    cfgTemp |= ((uint8) (cfgTseg2 << CANx_CFG_REG_TSEG2_SHIFT)) |
                               ((uint8) (sjw << CANx_CFG_REG_SJW_SHIFT));

                    /* write register byte 0 */
                    CANx_CFG_REG.byte[0u] = cfgTemp;

                    /* Verify 1st byte of CANx_CFG_REG register */
                    if (CANx_CFG_REG.byte[0u] == cfgTemp)
                    {
                        result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetRestartType
********************************************************************************
*
* Summary:
*  This function sets Reset type. Types of Reset are Automatic and Manual.
*  Manual Reset is recommended setting.
*
* Parameters:
*  reset: Reset Type.
*   Define                             Description
*    CANx_MANUAL_RESTART    After bus-off, the CAN must be restarted
*                                       by hand. This is the recommended 
*                                       setting.
*    CANx_AUTO_RESTART      After bus-off, the CAN is restarting 
*                                       automatically after 128 groups of 11 
*                                       recessive bits.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_SetRestartType(uint8 reset) 
{
    uint8 result = CANx_FAIL;

    if (reset == CANx_RESTART_BY_HAND)
    {
        CANx_CFG_REG.byte[0u] &= (uint8) (~CANx_RESET_MASK);   /* Manual reset */

        /* Verify that bit is cleared */
        if ((CANx_CFG_REG.byte[0u] & CANx_RESET_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }
    }
    else
    {
        CANx_CFG_REG.byte[0u] |= CANx_RESET_MASK;    /* Automatic restart */

        /* Verify that bit is set */
        if ((CANx_CFG_REG.byte[0u] & CANx_RESET_MASK) == CANx_RESET_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetEdgeMode
********************************************************************************
*
* Summary:
*  This function sets Edge Mode. Modes are 'R' to 'D'(Reccessive to Dominant)
*  and Both edges are used.
*
* Parameters:
*  edge: Edge Mode.
*   Define                             Description
*    CANx_EDGE_R_TO_D       Edge from R to D is used for 
*                                       synchronization
*    CANx_BOTH_EDGES        Both edges are used
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_SetEdgeMode(uint8 edge) 
{
    uint8 result = CANx_FAIL;

    if (edge == CANx_EDGE_R_TO_D)
    {
        /* Recessive to Dominant is used for synchronization */
        CANx_CFG_REG.byte[0u] &= (uint8) (~CANx_EDGE_MODE_MASK);

        /* Verify that bit is cleared */
        if ((CANx_CFG_REG.byte[0u] & CANx_EDGE_MODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                      
    }
    else
    {
        /* Both edges are to be used */
        CANx_CFG_REG.byte[0u] |= CANx_EDGE_MODE_MASK;

        /* Verify that bit is set */
        if ((CANx_CFG_REG.byte[0u] & CANx_EDGE_MODE_MASK) == CANx_EDGE_MODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetOpMode
********************************************************************************
*
* Summary:
*  This function sets Operation Mode. Operations Modes are Active of Listen
*  Only.
*
* Parameters:
*  opMode: Operation Mode value.
*   Define                             Description
*    CANx_ACTIVE_MODE       Active mode
*    CANx_LISTEN_ONLY       CAN listen only
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_SetOpMode(uint8 opMode) 
{
    uint8 result = CANx_FAIL;

    if (opMode == CANx_ACTIVE_MODE)
    {
        /* CAN in Active mode */
        CANx_CMD_REG.byte[0u] &= (uint8) (~CANx_OPMODE_MASK);

        /* Verify that bit is cleared */
        if ((CANx_CMD_REG.byte[0u] & CANx_OPMODE_MASK) == 0u)
        {
            result = CYRET_SUCCESS;
        }                
    }
    else
    {
        /* CAN listen only */
        CANx_CMD_REG.byte[0u] |= CANx_OPMODE_MASK;

        /* Verify that bit is set */
        if ((CANx_CMD_REG.byte[0u] & CANx_OPMODE_MASK) == CANx_OPMODE_MASK)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_RXRegisterInit
********************************************************************************
*
* Summary:
*  This function writes only receive CAN registers.
*
* Parameters:
*  regAddr: Pointer to CAN receive register;
*  config:  Value that will be written in register.
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*    CANx_OUT_OF_RANGE      Function parameter is out of range
*
*******************************************************************************/
uint8 CANx_RXRegisterInit(reg32 *regAddr, uint32 config)
                                      
{
    uint8 result = CANx_OUT_OF_RANGE;    
    
    if ((((uint32) regAddr & 0x0000FFFFu) >= ((uint32) CANx_RX_FIRST_REGISTER_PTR & 0x0000FFFFu)) &&
         ((((uint32) regAddr & 0x0000FFFFu)) <= ((uint32) CANx_RX_LAST_REGISTER_PTR & 0x0000FFFFu)))
    {
        result = CANx_FAIL;
        
        if ((((uint32) regAddr & 0x0000FFFFu) % CANx_RX_CMD_REG_WIDTH) == 0u)
        {
            config |= CANx_RX_WPN_SET;
            
    /* Disable isr */
    CyIntDisable(CANx_ISR_NUMBER);

            /* Write defined RX CMD registers */
            CY_SET_REG32(regAddr, config);
            
    /* Enable isr */
    CyIntEnable(CANx_ISR_NUMBER);

            /* Verify register */
            if ((CY_GET_REG32(regAddr) & CANx_RX_READ_BACK_MASK) ==
                (config & CANx_RX_READ_BACK_MASK))
            {
                result = CYRET_SUCCESS;
            }
        }
        /* All registers except RX CMD*/
        else
        {            
    /* Disable isr */
    CyIntDisable(CANx_ISR_NUMBER);

            /* Write defined CAN receive register */
            CY_SET_REG32(regAddr, config);
            
    /* Enable isr */
    CyIntEnable(CANx_ISR_NUMBER);

            /* Verify register */
            if (CY_GET_REG32(regAddr) == config)
            {
                result = CYRET_SUCCESS;
            }
        }
    }
    
    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_SetIrqMask
********************************************************************************
*
* Summary:
*  This function sets to enable/disable particular interrupt sources. Interrupt
*  Mask directly write to CAN Interrupt Enable register. A particular interrupt
*  source is enabled by setting its respective flag to 1.
*
* Parameters:
*  mask: Interrupt enable/disable request. 1 bit per interrupt source.
*   Define                                    Description
*    CANx_GLOBAL_INT_ENABLE        Global Interrupt Enable Flag
*    CANx_ARBITRATION_LOST_ENABLE  Arbitration Loss Interrupt Enable
*    CANx_OVERLOAD_ERROR_ENABLE    Overload Interrupt Enable
*    CANx_BIT_ERROR_ENABLE         Bit Error Interrupt Enable
*    CANx_STUFF_ERROR_ENABLE       Stuff Error Interrupt Enable
*    CANx_ACK_ERROR_ENABLE         Ack Error Interrupt Enable
*    CANx_FORM_ERROR_ENABLE        Form Error Interrupt Enable
*    CANx_CRC_ERROR_ENABLE         CRC Error Interrupt Enable
*    CANx_BUS_OFF_ENABLE           Busoff State Interrupt Enable
*    CANx_RX_MSG_LOST_ENABLE       Rx Msg Loss Interrupt Enable
*    CANx_TX_MESSAGE_ENABLE        Tx Msg Sent Interrupt Enable
*    CANx_RX_MESSAGE_ENABLE        Msg Recived Interrupt Enable
*
* Return:
*  Indication whether register is written and verified.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_SetIrqMask(uint16 mask) 
{
    uint8 result = CANx_FAIL;

    /* Write byte 0 and 1 of CANx_INT_EN_REG register */
    CY_SET_REG16((reg16 *) &CANx_INT_EN_REG, mask);

    /* Verify CANx_INT_EN_REG register */
    if (CY_GET_REG16((reg16 *) &CANx_INT_EN_REG) == mask)
    {
        result = CYRET_SUCCESS;
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_GetTXErrorflag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of TX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 CANx_GetTXErrorFlag(void) 
{
    /* Get the state of the transmit error flag */
    return (((CANx_ERR_SR_REG.byte[2u] & CANx_TX_ERROR_FLAG_MASK) ==
              CANx_TX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_GetRXErrorFlag
********************************************************************************
*
* Summary:
*  This function returns the bit that indicates if the number of RX errors
*  exceeds 0x60.
*
* Parameters:
*  None.
*
* Return:
*  Indication whether the number of TX errors exceeds 0x60.
*
*******************************************************************************/
uint8 CANx_GetRXErrorFlag(void) 
{
    /* Get the state of the receive error flag */
    return (((CANx_ERR_SR_REG.byte[2u] & CANx_RX_ERROR_FLAG_MASK) ==
              CANx_RX_ERROR_FLAG_MASK) ? 1u : 0u);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_TXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Transmit Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Transmit Errors.
*
*******************************************************************************/
uint8 CANx_GetTXErrorCount(void) 
{
    /* Get the state of the transmit error count */
    return (CANx_ERR_SR_REG.byte[0u]);    /* bits 7-0 */
}


/*******************************************************************************
* FUNCTION NAME:   CANx_RXErrorCount
********************************************************************************
*
* Summary:
*  This function returns the number of Receive Errors.
*
* Parameters:
*  None.
*
* Return:
*  Number of Receive Errors.
*
*******************************************************************************/
uint8 CANx_GetRXErrorCount(void) 
{
    /* Get the state of the receive error count */
    return (CANx_ERR_SR_REG.byte[1u]);    /* bits 15-8 */
}


/*******************************************************************************
* FUNCTION NAME:   CANx_GetErrorState
********************************************************************************
*
* Summary:
*  This function returns error status of CAN Component.
*
* Parameters:
*  None.
*
* Return:
*  Error status.
*
*******************************************************************************/
uint8 CANx_GetErrorState(void) 
{
    /* Get the error state of the receiver */
    return (CANx_ERR_SR_REG.byte[2u] & CANx_ERROR_STATE_MASK);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_RxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all receive registers for particular mailbox.
*
* Parameters:
*  rxConfig: Pointer to structure that contain all required values to configure
*  all receive registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_RxBufConfig(const CANx_RX_CFG *rxConfig)
                                   
{
    uint8 result = CANx_FAIL;

    /* Write RX CMD Register */
    CY_SET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxcmd, (rxConfig->rxcmd |
                 CANx_RX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxcmd) &
         CANx_RX_READ_BACK_MASK) == (rxConfig->rxcmd & CANx_RX_WPN_CLEAR))
    {
        /* Write RX AMR Register */
        CY_SET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxamr, rxConfig->rxamr);
        if (CY_GET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxamr) == rxConfig->rxamr)
        {
            /* Write RX ACR Register */
            CY_SET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxacr, rxConfig->rxacr);
            if (CY_GET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxacr) == rxConfig->rxacr)
            {
                /* Write RX AMRD Register */
                CY_SET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxamrd, 0xFFFFFFFFu);
                if (CY_GET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxamrd) == 0xFFFFFFFFu)
                {
                    /* Write RX ACRD Register */
                    CY_SET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxacrd, 0x00000000u);
                    if (CY_GET_REG32((reg32 *) &CANx_RX[rxConfig->rxmailbox].rxacrd) == 0x00000000u)
                    {
                            result = CYRET_SUCCESS;
                    }
                }
            }
        }
    }

    return (result);
}


/*******************************************************************************
* FUNCTION NAME:   CANx_TxBufConfig
********************************************************************************
*
* Summary:
*  This function configures all transmit registers for particular mailbox.
*  Mailbox number contains CANx_TX_CFG structure.
*
* Parameters:
*  txConfig: Pointer to structure that contain all required values to configure
*  all transmit registers for particular mailbox.
*
* Return:
*  Indication if particular configuration has been accepted or rejected.
*   Define                             Description
*    CYRET_SUCCESS                      Function passed successfully
*    CANx_FAIL              Function failed
*
*******************************************************************************/
uint8 CANx_TxBufConfig(const CANx_TX_CFG *txConfig) 
                                   
{
    uint8 result = CANx_FAIL;

    /* Write TX CMD Register */
    CY_SET_REG32((reg32 *) &CANx_TX[txConfig->txmailbox].txcmd, (txConfig->txcmd |
                 CANx_TX_WPN_SET));
    if ((CY_GET_REG32((reg32 *) & CANx_TX[txConfig->txmailbox].txcmd) &
        CANx_TX_READ_BACK_MASK) == (txConfig->txcmd & CANx_TX_WPN_CLEAR))
    {
        /* Write TX ID Register */
        CY_SET_REG32( (reg32 *) &CANx_TX[txConfig->txmailbox].txid, txConfig->txid);
        if (CY_GET_REG32( (reg32 *) &CANx_TX[txConfig->txmailbox].txid) == txConfig->txid)
        {
            result = CYRET_SUCCESS;
        }
    }

    return (result);
}


/* [] END OF FILE */
