/*******************************************************************************
* File Name: CANx_PM.c
* Version 2.30
*
* Description:
*  This file contains the setup, control and status commands to support 
*  component operations in low power mode.  
*
* Note:
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CANx.h"

static CANx_BACKUP_STRUCT CANx_backup = 
{ 0u,
  0u,
  CANx_INIT_INTERRUPT_MASK,
  CANx_MODE_MASK,
  (((uint32) ((uint32) CANx_SYNC_EDGE     << CANx_EDGE_MODE_SHIFT))     | 
   ((uint32) ((uint32) CANx_SAMPLING_MODE << CANx_SAMPLE_MODE_SHIFT))   |
   ((uint32) ((uint32) CANx_CFG_REG_SJW   << CANx_CFG_REG_SJW_SHIFT))   | 
   ((uint32) ((uint32) CANx_RESET_TYPE    << CANx_RESET_SHIFT))         |
   ((uint32) ((uint32) CANx_CFG_REG_TSEG2 << CANx_CFG_REG_TSEG2_SHIFT)) |
   ((uint32) ((uint32) CANx_CFG_REG_TSEG1 << CANx_CFG_REG_TSEG1_SHIFT)) |
   ((uint32) ((uint32) CANx_ARBITER       << CANx_ARBITER_SHIFT))       |
   ((uint32) ((uint32) CANx_BITRATE       << CANx_BITRATE_SHIFT))) };


/*******************************************************************************
* Function Name: CANx_SaveConfig
********************************************************************************
*
* Summary:
*  Save CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
* 
* Global Variables:
*  CANx_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CANx_SaveConfig(void) 
{
    CANx_backup.intSr = (CY_GET_REG32(CANx_INT_SR_PTR));
    CANx_backup.intEn = (CY_GET_REG32(CANx_INT_EN_PTR));
    CANx_backup.cmd = (CY_GET_REG32(CANx_CMD_PTR));
    CANx_backup.cfg = (CY_GET_REG32(CANx_CFG_PTR));
}


/*******************************************************************************
* Function Name: CANx_RestoreConfig
********************************************************************************
*
* Summary:
*  Restore CAN configuration.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  CANx_backup - used when non-retention registers are restored.
*
* Side Effects:
*  If this API is called without first calling SaveConfig then in the following
*  registers will be default values from Customizer: CANx_INT_SR, 
*  CANx_INT_EN, CANx_CMD, CANx_CFG.
*
*******************************************************************************/
void CANx_RestoreConfig(void) 
{
    CY_SET_REG32(CANx_INT_SR_PTR, CANx_backup.intSr);
    CY_SET_REG32(CANx_INT_EN_PTR, CANx_backup.intEn);
    CY_SET_REG32(CANx_CMD_PTR, CANx_backup.cmd);
    CY_SET_REG32(CANx_CFG_PTR, CANx_backup.cfg);
}


/*******************************************************************************
* Function Name: CANx_Sleep
********************************************************************************
* 
* Summary:
*  Prepare CAN Component goes to sleep.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  CANx_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CANx_Sleep(void) 
{   
    if (0u != (CY_GET_REG32(CANx_CMD_PTR) & CANx_MODE_MASK))
    {
        CANx_backup.enableState = 1u;
    }
    else /* The CAN block is disabled */
    {
        CANx_backup.enableState = 0u;
    }
    
    CANx_SaveConfig();
    (void) CANx_Stop();
}


/*******************************************************************************
* Function Name: CANx_Wakeup
********************************************************************************
* 
* Summary:
*  Prepare CAN Component to wake up.
*
* Parameters:  
*  None.
*
* Return: 
*  None.
*
* Global Variables:
*  CANx_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void CANx_Wakeup(void) 
{           
    /* Enable power to CAN */
    CANx_PM_ACT_CFG_REG |= CANx_ACT_PWR_EN;
    
    CANx_RestoreConfig();
    
    if (CANx_backup.enableState != 0u)
    {
        /* Enable component's operation */
        (void) CANx_Enable();
    } /* Do nothing if component's block was disabled before */
}


/* [] END OF FILE */
